/* -----------------------------------------------------------------------------
*
*  nppio.c
*
*  ANTz - realtime 3D data visualization tools for the real-world, based on NPE.
*
*  ANTz is hosted at http://openantz.com and NPE at http://neuralphysics.org
*
*  Written in 2010-2012 by Shane Saxon - contact@saxondigital.net
*
*  Please see main.c for a complete list of additional code contributors.
*
*  To the extent possible under law, the author(s) have dedicated all copyright 
*  and related and neighboring rights to this software to the public domain
*  worldwide. This software is distributed without any warranty.
*
*  Released under the CC0 license, which is GPL compatible.
*
*  You should have received a copy of the CC0 Public Domain Dedication along
*  with this software (license file named COPYING.txt). If not, see
*  http://creativecommons.org/publicdomain/zero/1.0/
*
* --------------------------------------------------------------------------- */

#include "npio.h"

#include "io/npfile.h"
#include "io/npch.h"
#include "io/npmouse.h"
#include "io/npconsole.h"
#include "io/db/npmysql.h"

//-----------------------------------------------------------------------------
void npInitIO (void* dataRef)
{
	struct dbNewConnect *connect = malloc(sizeof(struct dbNewConnect*)); // debug db //zzsql
	npInitDbConnection(connect, MySQL, "127.0.0.1", "root", "admin", "test");
	npOpenDbConnect(connect, dataRef);
	npAttachMySQLtoDataRef(connect, dataRef); // debug db // This can be generalized later, debug db
	//zzsql

	npInitMouse (dataRef);
	npInitFile (dataRef);			// may update default init from pref file
	npInitCh (dataRef);	//zz-JJ
}


//-----------------------------------------------------------------------------
void npCloseIO (void* dataRef)
{
	npCloseFile (dataRef);
	npCloseMouse (dataRef);
	npCloseCh (dataRef);	//zz-JJ
}


// update IO devices such as keyboard, mice, video, TNG3B and other inputs
//------------------------------------------------------------------------------
void npUpdateIO (void* dataRef)
{
	pData data = (pData) dataRef;

	data->io.cycleCount++;

	//we double buffer the mouse delta movement to maintain engine cycle sync
	npUpdateMouse (dataRef);
	
	npUpdateConsole (dataRef);

	npUpdateCh (dataRef);	//zz-JJ
}


//view tree of files catalogued
//------------------------------------------------------------------------------
void npViewFiles (void* dataRef)
{
	FILE* file = NULL;

	pData data = (pData) dataRef;

	//display names and paths of indexed files

	//let user find and choose the files
//	npFileBrowser (dataRef);
}

//void npdbLoadByName (int item, void* dataRef);

//zz debug, test function can be used as a template
void npdbLoadMenuItem (int menuItem, void* dataRef);
//------------------------------------------------------------------------------
void npdbLoadMenuItem (int menuItem, void* dataRef)
{
	char msg[4096];

	pData data = (pData) dataRef;											//zzsql
	char *selectedItem = NULL;
	//pNPmenu menu = data->io.gl.hud.console.menu;
	struct databaseObject *dbResult = NULL;
	pNPdatabases *dbList = ((struct dbNewConnect*)data->io.connect)->dbList;
	
	selectedItem = ((struct pNPdatabases*)dbList)->list[menuItem];
	
	
	//load database by item index using same list that generated the menu
//	sprintf (msg, "Loading: %d", item);
//	npPostMsg (msg, kNPmsgView, dataRef);

	printf("\n-----YOU SELECTED %s-----\n", selectedItem);
	
	npdbCtrl(data->io.connect, DATABASE, NULL, selectedItem, NULL, NULL, NULL, NULL, Use, dataRef);
	
	dbResult = npdbCtrl(data->io.connect, TABLE, NODE_STATE_TABLE, "nodestatetable", NULL, NULL, NULL, NULL, Select, dataRef);
	
	//dbTagsResult = npdbCtrl(connect, TABLE, TAGS_TABLE, "tagsTable", NULL, NULL, NULL, NULL, Select, dataRef);
	
	npNewLoadNodeStateResultIntoAntz(dbResult->dbObject, dataRef);			//zzsql

	npSelectNode(data->map.node[kNPnodeRootPin], data);
	npPostMsg("Done Loading Database", kNPmsgDB, data);
}

//zz debug, test function can be used as a template
pNPmenu npdbGetMenu (void* dataRef);
//------------------------------------------------------------------------------
pNPmenu npdbGetMenu (void* dataRef)
{
	int i = 0;
	int itemCount = 50;		//arbitrary

	char name[256] = "antz1209111652\0";
	char path[kNPmaxPath] = "127.0.0.1\0";

	pNPdatabases dbList = NULL;
	pNPmenu menu = NULL;

	pData data = (pData) dataRef;

	//get current list of databases and store it in the global data struct
//	dbList = npdbGetList(dataRef);
//	data->io.db.list = dbList;
	dbList = npdbGetList(dataRef);											//zzsql
	((struct dbNewConnect*)data->io.connect)->dbList = (struct pNPdatabases*)dbList; //zzsql
	
	menu = malloc(sizeof(NPmenu));
	if (!menu)
	{
		npPostMsg("err 2255 - malloc failed npdbGetMenu", kNPmsgErr, dataRef);
		return NULL;
	}
	
	itemCount = ((struct dbNewConnect*)data->io.connect)->dbList->size;		//zzsql
	menu->list = malloc(sizeof(char*) * itemCount);

	//set the menubar fields
	menu->name = npNewStrcpy("DB Name", data);
	menu->details = npNewStrcpy("Lccation", data);

	//copy the db list to the menu format
	for (i=1; i <= itemCount; i++)
	{
		menu->list[i] = malloc(80 + 1);
		if (!menu)
		{
			npPostMsg("err 2256 - malloc failed npdbGetMenu", kNPmsgErr, dataRef);
			return NULL;
		}
		sprintf (menu->list[i], "|%4.0d.| %16s | %40s |", i, ((struct dbNewConnect*)data->io.connect)->dbList->list[i], path); //zzsql
//		sprintf (menu->list[i], "|%4.0d.| %16s | %40s |", i, name, path);	//zzsql
		//dbList->list[i].name, dbList->list[i].details
	}

	//set the list size
	menu->count = itemCount;

	return menu;
}

//view catalog of DBs from pathmap
//------------------------------------------------------------------------------
void npViewDatabases (void* dataRef)
{
	pNPmenu menuList = NULL;

	pData data = (pData) dataRef;
	pNPconsole console = &data->io.gl.hud.console;

	menuList = malloc(sizeof(NPmenu));

	console->menu = npdbGetMenu(dataRef);
	if (console->menu == NULL)
		return;					//failed to get menu

	//call menu function, pass list ptr and callback function ptr
	npConsoleMenu (npdbLoadMenuItem, console, dataRef);
}

//hybrid ASCII text based console(s) with 3D objects, xtree vr
//choose a folder, file or DB to view, can select multiple items
//manipulating 3D objects changes console text and simultaneously vice versa
//supports multiple instances of the viewer and its components
//selection sets and active object tracking per console, multi-user conpatible
//xtree gold style UI with wildcards and key shortcuts... user prompts
//commands - Load, Save, Cut, Copy, Paste, Delete, Split, Merge, Move, Convert
//select - drag region, shift/ctrl/arrows, wildcard and boolean expressions
//------------------------------------------------------------------------------
void npViewer (void* dataRef)
{
	FILE* file = NULL;

	pData data = (pData) dataRef;

//	if (data->io.gl.fullscreen)	//workaround, gl file dialog conflict
//		npCtrlCommand(kNPcmdFullscreen, data);

	//dynamic components, default has Files on left and DBs on the right
//	npViewFiles(dataRef);			//view tree of files catalogued
	npViewDatabases(dataRef);		//view catalog of DBs from pathmap

	//npViewDataMap()		//can be used for runtime code profiling analysis
	//npViewIO()
	//npViewNetworks()
	//npViewSystem()		//shows HW and SW components, IO, RAM, CPU, PCIe
	//npViewCode()			//code editor with syntax highlighting and compile
							//automatic dynamic plugin loading within antz
	
//	npPostTool (kNPtoolBrowse, data);
	
	//npThread(npDataBrowser, dataRef);

	//currently display a simple list or to launch a file dialog
	//later add full 3D xtree viewer with live data sampling and thumb images...
	// npDisplayDatasets()	//known folders on the left and known DB on the right
	//						//option to specify or browse for new folder or DB
							//options to filter data type, 
								// meta info (date, size, etc.)
								// file types, DB, CSV, JPG...
	// npCtrlCommand(kNPcmdPauseUpdates, dataRef);		//freeze NPE, GL, etc...
	// if (file) Then
	//  file = npFileDialog (data->io.file.csvPath, kNPfileDialogOpen, dataRef);
	//	if (file == directory || file.tableType == tablemap)
	//   do the following in a loop for all files... CSV, JPG...
	//		npFileOpen (file,dataRef);
	//		format = npGetFileFormat (file, dataRef);
	//		dataType = npGetDataType (file, dataRef);
	//		npMultiThread (npParse(fileBuffer,dataRef));
	//		do npReadFileChunk()
	// else (DB)
	//	npListDB();	//list known databases and allow user to choose (multiple)
	//  db = npBrowseDB(dataRef);
	//  npOpenDB (db, dataRef);
	//	
	// sync 
	// npCtrlCommand(kNPcmdDoUpdates, dataRef);	//update tracks, NPE, GL...
	// return 0;


	return;
}



